-- RBT-TEST - Проверка установки меток на файлы отношений
-- Проверяется соответствие метки на таблице базы данных и на всех ее файлах в файловой системе

-- init-config-tools.sql
--\i /usr/share/postgresql/11/test/pgacext/support/init-config-tools.sql

\connect postgres postgres - 6002

\i 'support/init.sql'
\i 'support/init-mac.sql'

-- Создаем тестовую таблицу
CREATE TABLE test_table_1 (id INTEGER PRIMARY KEY, txt TEXT);

-- Ожидается, что начальная метка файла таблицы {0,0}, существуют файл уникального основного индекса и TOAST
-- Ожидается:
-- Основной файл - есть, метка {0,0}
-- Файл _fsm - нет
-- Файл _vm - нет
-- Дополнительный .1 - нет
-- Индекс - есть, метка {0,0}
-- TOAST - есть, метка {0,0}
\! sudo bash support/check_table.sh test_table_1

MAC LABEL ON TABLE test_table_1 IS '{2,0}';

-- Ожидается, что метка файла таблицы {2,0},
-- существуют файл уникального основного индекса и TOAST
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - нет
-- Файл _vm - нет
-- Дополнительный .1 - нет
-- Индекс - есть, метка {2,0}
-- TOAST - есть, метка {2,0}
\! sudo bash support/check_table.sh test_table_1

-- Создаем вторую тестовую таблицу для проверки метки на дополнительных файлах таблицы
CREATE TABLE test_table_2 (id INTEGER, data INTEGER);
MAC LABEL ON TABLE test_table_2 IS '{2,0}';

-- Проверка сразу после создания таблицы
-- Ожидается, что метка файла таблицы {2,0}
-- а дополнительных файлов нет
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - нет
-- Файл _vm - нет
-- Дополнительный .1 - нет
-- Индекс - нет
-- TOAST - нет
\! sudo bash support/check_table.sh test_table_2

-- Добавим записи в таблицу
INSERT INTO test_table_2 (id, data) VALUES (10000200, 9), ( 10000300, 10);

-- Проверка после добавления записей в таблицу
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - нет
-- Файл _vm - нет
-- Дополнительный .1 - нет
-- Индекс - нет
-- TOAST - нет
\! sudo bash support/check_table.sh test_table_2

--ALTER TABLE test_table_2 ADD COLUMN txt VARCHAR(8193); -- 8192 = max к-во на странице
ALTER TABLE test_table_2 ADD COLUMN txt TEXT;
-- Проверка после добавления записей в таблицу
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - нет
-- Файл _vm - нет
-- Дополнительный .1 - нет
-- Индекс - нет
-- TOAST - есть, метка {2,0}
\! sudo bash support/check_table.sh test_table_2

-- Заполнение таблицы данными > 1 Гб для создания дополнительного файла. По-умолчанию PostgreSQL хранит данные в файлах по 1 Гб.
INSERT INTO test_table_2 (id, txt) SELECT id, 'fjpasodifhgapdifj023jfpофваттйзушп02345ваьрapc.vnbngya[d0322ffjfjflllll' FROM generate_series(1, 10000000) id;

-- Проверка размера таблицы
-- Ожидается 1, если размер таблицы больше 1ГБ
select case when pg_relation_size('test_table_2', 'main') > 1073741824 then 1 else 0 end;

-- Проверка после наполнения таблицы
-- должен появиться дополнительный файл, т.к. превышен лимит в 1 ГБ;
-- так же за время наполнения таблицы был создан файл FSM
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - есть, метка {2,0}
-- Файл _vm - нет
-- Дополнительный .1 - есть, метка {2,0}
-- Индекс - нет
-- TOAST - есть, метка {2,0}
\! sudo bash support/check_table.sh test_table_2

-- Удаление нескольких записей
DELETE FROM test_table_2 WHERE id = 99999 or id = 999999;

-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - есть, метка {2,0}
-- Файл _vm - нет
-- Дополнительный .1 - есть, метка {2,0}
-- Индекс - нет
-- TOAST - есть, метка {2,0}
\! sudo bash support/check_table.sh test_table_2

-- Создаем индекс на поле id таблицы test_table_2
CREATE UNIQUE INDEX test_table_2_pkey ON test_table_2 (id);

-- Проверка после создания индекса на таблице test_table_2
-- должен появиться файл индекса с меткой {2,0}
-- Проверка после удаления записи из таблицы test_table_2
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - есть, метка {2,0}
-- Файл _vm - нет
-- Дополнительный .1 - есть, метка {2,0}
-- Индекс - есть, метка {2,0}
-- TOAST - есть, метка {2,0}
\! sudo bash support/check_table.sh test_table_2

-- Изменение метки таблицы без VACUUM
MAC LABEL ON TABLE test_table_2 IS '{1,1}';

-- Проверка метки таблицы без VACUUM после изменения метки
-- Все файлы таблицы должны иметь метку {1,1}
-- Ожидается:
-- Основной файл - есть, метка {1,1}
-- Файл _fsm - есть, метка {1,1}
-- Файл _vm - нет
-- Дополнительный .1 - есть, метка {1,1}
-- Индекс - есть, метка {1,1}
-- TOAST - есть, метка {1,1}
\! sudo bash support/check_table.sh test_table_2

-- После VACUUM инициализируется карта видимости - visibility map - vm
VACUUM test_table_2;

-- Проверка после VACUUM
-- Ожидается:
-- Основной файл - есть, метка {1,1}
-- Файл _fsm - есть, метка {1,1}
-- Файл _vm - есть, метка {1,1}
-- Дополнительный .1 - есть, метка {1,1}
-- Индекс - есть, метка {1,1}
-- TOAST - есть, метка {1,1}
\! sudo bash support/check_table.sh test_table_2

-- Изменение мандатной метки таблицы test_table_2 на {2,0}
MAC LABEL ON TABLE test_table_2 IS '{2,0}';

-- Проверка после изменения метки таблицы test_table_2 на {2,0}
-- Ожидается:
-- Основной файл - есть, метка {2,0}
-- Файл _fsm - есть, метка {2,0}
-- Файл _vm - есть, метка {2,0}
-- Дополнительный .1 - есть, метка {2,0}
-- Индекс - есть, метка {2,0}
-- TOAST - есть, метка {2,0}
\! sudo bash support/check_table.sh test_table_2

-- Изменение мандатной метки таблицы test_table_2 на {1,2}
MAC LABEL ON TABLE test_table_2 IS '{1,2}';

-- Проверка после изменения метки таблицы test_table_2 на {1,2}
-- Ожидается:
-- Основной файл - есть, метка {1,2}
-- Файл _fsm - есть, метка {1,2}
-- Файл _vm - есть, метка {1,2}
-- Дополнительный .1 - есть, метка {1,2}
-- Индекс - есть, метка {1,2}
-- TOAST - есть, метка {1,2}
\! sudo bash support/check_table.sh test_table_2

-- Проверка изменения метки табличного пространства и файлов таблицы
-- при изменении табличного пространства
create tablespace mac_test_tblspc LOCATION '/pg_test_tablespace';

alter table test_table_2 set tablespace mac_test_tblspc;
alter index test_table_2_pkey set tablespace mac_test_tblspc;
-- Ожидается:
-- Основной файл - есть, метка {1,2}
-- Файл _fsm - есть, метка {1,2}
-- Файл _vm - есть, метка {1,2}
-- Дополнительный .1 - есть, метка {1,2}
-- Индекс - есть, метка {1,2}
-- TOAST - нет, т.к. находится в другом табличном пространстве
\! bash support/check_table.sh test_table_2 /pg_test_tablespace

alter table test_table_2 set tablespace pg_default;
alter index test_table_2_pkey set tablespace pg_default;
-- Проверка после изменения табличного пространства
-- Ожидается:
-- Основной файл - есть, метка {1,2}
-- Файл _fsm - есть, метка {1,2}
-- Файл _vm - есть, метка {1,2}
-- Дополнительный .1 - есть, метка {1,2}
-- Индекс - есть, метка {1,2}
-- TOAST - есть, метка {1,2}
\! bash support/check_table.sh test_table_2

-- select oid, relfilenode, relname from pg_class where oid = (select reltoastrelid from pg_class where relname = 'test_table_2');
-- select oid, reltoastrelid, relfilenode from pg_class where relname = 'test_table_2';

-- select * from test_table_2 where id = 8999999;
-- insert into test_table_2 (id, txt) values (11000000, 'test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_test_');
-- select * from test_table_2 where id = 11000000;

drop tablespace mac_test_tblspc;

MAC LABEL ON TABLESPACE pg_default IS '{3,2}';

\i 'support/done.sql'
